__all__ = ['OutgoingChunk', 'IncomingChunk', 'Outgoing', 'Incoming', 'IncomingJson', 'OutgoingJson']

import os
import errno
import socket
import struct
import json

HEADER_FORMAT = '!L' # message length
HEADER_LENGTH = struct.calcsize(HEADER_FORMAT)

class OutgoingChunk:

    def __init__(self, chunk):
        self.chunk = chunk
        self.offset = 0

    def send(self, sock):
        assert self.offset < len(self.chunk), 'Message already sent.'
        while self.offset < len(self.chunk):
            # assumed socket.send raises error when connection is closed
            self.offset += sock.send(self.chunk[self.offset:])

class IncomingChunk:

    def __init__(self, length):
        self.chunk = bytearray(length)
        self.offset = 0

    def recv(self, sock):
        assert self.offset < len(self.chunk), 'Message already received.'
        while self.offset < len(self.chunk):
            received = sock.recv_into(memoryview(self.chunk)[self.offset:], len(self.chunk) - self.offset, socket.MSG_WAITALL)
            if received == 0:
                raise socket.error(errno.ECONNRESET, os.strerror(errno.ECONNRESET))
            else:
                self.offset += received
        return self.chunk

class Outgoing:

    def __init__(self, message):
        self.message = message
        self.header_chunk = OutgoingChunk(struct.pack(HEADER_FORMAT, len(message)))
        self.message_chunk = None

    def send(self, sock):
        if self.message_chunk is None:
            self.header_chunk.send(sock)
            self.message_chunk = OutgoingChunk(self.message)
        self.message_chunk.send(sock)

class Incoming:

    def __init__(self):
        self.header_chunk = IncomingChunk(HEADER_LENGTH)
        self.length = None
        self.message_chunk = None

    def recv(self, sock):
        if self.length is None:
            (self.length,) = struct.unpack(HEADER_FORMAT, self.header_chunk.recv(sock))
        if self.message_chunk is None:
            self.message_chunk = IncomingChunk(self.length)
        return self.message_chunk.recv(sock)

class OutgoingJson:

    def __init__(self, object):
        self.outgoing_message = Outgoing(json.dumps(object).encode('utf-8'))

    def send(self, sock):
        self.outgoing_message.send(sock)

class IncomingJson:

    def __init__(self):
        self.incoming_message = Incoming()

    def recv(self, sock):
        return json.loads(self.incoming_message.recv(sock).decode('utf-8'))
