import socket
import threading
import sockmsg
import sys
import atexit
from contextlib import closing

BACKLOG = 5
SERVER_NAME = 'server-name-not-set'

class ClientCommandError(Exception):
    pass

clients = {}
clients_lock = threading.Lock()

def main():
    global SERVER_NAME
    SERVER_NAME = sys.argv[1]
    atexit.register(close_clients)
    server_socket = create_server_socket(sys.argv[2], int(sys.argv[3]))
    with closing(server_socket):
        accept_clients(server_socket)

def close_clients():
    with clients_lock:
        for target_socket in clients.values():
            target_socket.close()

def create_server_socket(server_host, server_port):
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind((server_host, server_port))
    server_socket.listen(BACKLOG)
    return server_socket

def accept_clients(server_socket):
    while True:
        client_socket, client_address = server_socket.accept()
        handle_client_connect(client_socket)

def handle_client_connect(client_socket):
    try:
        client_name = recv_client_name(client_socket)
    except ClientCommandError:
        client_socket.close()
        return
    with clients_lock:
        if client_name in clients:
            send_client_name_taken(client_socket)
            client_socket.close()
        else:
            send_client_accepted(client_socket)
            service_client(client_name, client_socket)

def recv_client_name(client_socket):
    ident_command = sockmsg.IncomingJson().recv(client_socket)
    if not valid_ident_command(ident_command):
        raise ClientCommandError
    else:
        return ident_command[1]

def send_client_name_taken(client_socket):
    sockmsg.OutgoingJson(['TAKEN']).send(client_socket)

def send_client_accepted(client_socket):
    sockmsg.OutgoingJson(['ACCEPTED', SERVER_NAME]).send(client_socket)

def valid_ident_command(command):
    return type(command) == list and \
            len(command) == 2 and \
            command[0] == 'IDENT' and \
            type(command[1]) == str

def service_client(client_name, client_socket):
    print(client_name, 'connected')
    clients[client_name] = client_socket
    client_thread = setup_client_thread(client_name, client_socket)
    client_thread.start()

def setup_client_thread(client_name, client_socket):
    client_thread = threading.Thread(target=client_communicator, args=(client_name, client_socket))
    client_thread.daemon = True
    return client_thread

def client_communicator(client_name, client_socket):
    while True:
        try:
            message = sockmsg.IncomingJson().recv(client_socket)
            print(client_name, message)
            with clients_lock:
                for target, target_socket in clients.items():
                    if target != client_name:
                        sockmsg.OutgoingJson([client_name, message]).send(target_socket)
        except socket.error:
            with clients_lock:
                client_socket.close()
                del clients[client_name]
                print(client_name, 'left')
                break

if __name__ == '__main__':
    main()
